package com.ibm.oti.shared;
/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 1998, 2010  All Rights Reserved
 */

/**
 * Implementation of SharedClassTokenHelper.
 * <p>
 * @author		OTI
 * @version		initial
 *
 * @see SharedClassTokenHelper
 * @see SharedClassHelperFactory
 * @see SharedClassAbstractHelper
 */
public class SharedClassTokenHelperImpl extends SharedClassAbstractHelper implements SharedClassTokenHelper {
	/* Not public - should only be created by factory */
	SharedClassTokenHelperImpl(ClassLoader loader, int id, boolean canFind, boolean canStore) {
		initialize(loader, id, canFind, canStore);
		initializeShareableClassloader(loader);
	}

	private native boolean findSharedClassImpl2(int id, String className, ClassLoader loader, String token,
			boolean doFind, boolean doStore, byte[] romClassCookie);

	private native boolean storeSharedClassImpl2(int id, ClassLoader loader, String token, Class clazz, byte[] nativeFlags);

	public synchronized byte[] findSharedClass(String token, String className) {
		if (!canFind) {
			return null;
		}
		if (token==null) {
			printVerboseError("Cannot call findSharedClass with null token. Returning null.");
			return null;
		}
		if (className==null) {
			printVerboseError("Cannot call findSharedClass with null class name. Returning null.");
			return null;
		}
		SharedClassFilter theFilter = getSharingFilter();
		boolean doFind, doStore;
		if (theFilter!=null) {
			doFind = theFilter.acceptFind(className);
			/* Don't invoke the store filter if the cache is full */
			if (nativeFlags[CACHE_FULL_FLAG] == 0) {
				doStore = theFilter.acceptStore(className);
			} else {
				doStore = true;
			}
		} else {
			doFind = true;
			doStore = true;
		}
		byte[] romClassCookie = new byte[ROMCLASS_COOKIE_SIZE];
		boolean found = findSharedClassImpl2(this.id, className, getClassLoader(), token, doFind, doStore, romClassCookie);
		if (!found) {
			return null;
		}
		return romClassCookie;
	}

	public synchronized boolean storeSharedClass(String token, Class clazz) {
		if (!canStore) {
			return false;
		}
		if (token==null) {
			printVerboseError("Cannot call storeSharedClass with null token. Returning false.");
			return false;
		}
		if (clazz==null) {
			printVerboseError("Cannot call storeSharedClass with null Class. Returning false.");
			return false;
		}
		ClassLoader actualLoader = getClassLoader();
		if (!validateClassLoader(actualLoader, clazz)) {
			/* Attempt to call storeSharedClass with class defined by a different classloader */
			return false;
		}
		return storeSharedClassImpl2(this.id, actualLoader, token, clazz, nativeFlags);
	}

	String getHelperType() {
		return "SharedClassTokenHelper";
	}
}
